package uk.ac.leeds.ccg.gml;

import uk.ac.leeds.ccg.geotools.*;

import org.apache.xerces.parsers.*;
import org.w3c.dom.*;

import java.util.*;
import java.net.*;

public class GMLReader
{
    DOMParser parser;
    
    PolygonLayer layer=new PolygonLayer();
    SimpleGeoData descriptions = new SimpleGeoData();
    
    /**
     * Build a list of features from GML.
     * There is a security
     * constraint in java applets which prevents an applet from accessing a
     * URL different to the server where the applet came from.  So if you want
     * to access a Web Feature Server from somewhere on the web, you need to set
     * up a local proxy.  A local proxy sits on the server where the applet
     * came from, it recieves calls from the applet, forwards them onto the
     * Web Feature Server, and then funnels the responses back to the applet.
     * <br>If you are not using an applet, or if the URL of the GML is on the
     * same server as the applet, then you can set <i>proxy</i> to null.
     * @param url URL of the GML features
     * @param proxy URL of the local proxy server.
     */
    public GMLReader(URL url, URL proxy){
        try{
            parser = new DOMParser();
            if(proxy==null){
                //parser.setFeature(
                //    "http://xml.org/sax/features/validation", true);
                //parser.setFeature(
                //    "http://apache.org/xml/features/continue-after-fatal-error"
                //    ,true);
                parser.parse(url.toString());
            }else{
                //parser.setFeature(
                //    proxy.toString()
                //    +"?ProxyURL="
                //    +"http://xml.org/sax/features/validation"
                //    ,true);
                //parser.setFeature(
                //    proxy.toString()
                //    +"?ProxyURL="
                //    +"http://apache.org/xml/features/continue-after-fatal-error"
                //    ,true);
                parser.parse(
                    proxy.toString()
                    +"?ProxyURL="
                    +url.toString());
            }
            Document d = parser.getDocument();
            
            //FEATURES
            NodeList features = d.getElementsByTagName("Feature");
            for(int i=0;i<features.getLength();i++){
                Node node = features.item(i);
                parseFeature(node);
            }
        }
        catch(Exception e){System.out.println("Opps"+e);}
    }
    
    public GMLReader(URL url){
        this(url,null);
    }

    public Theme getTheme(){
        Theme t = new Theme(layer);
        t.setTipData(descriptions);
        return t;
       
    }
    
    public GeoData getDescriptions(){
        return descriptions;
    }
    
    public ShapeLayer getLayer(){
        return layer;
    }
    
    public void parseFeature(Node node){
         NamedNodeMap att= node.getAttributes();
         System.out.println(att.getNamedItem("featureType"));
         System.out.println(att.getNamedItem("fid"));
         int id = Integer.valueOf((att.getNamedItem("fid").getNodeValue().toString())).intValue();
         
         
         NodeList children = node.getChildNodes();
         for(int i=0;i<children.getLength();i++){
             Node child = children.item(i);
             if(child.getNodeName().equalsIgnoreCase("Polygon")){
                layer.addGeoPolygon(parsePolygon(id,child));
             }                                        
             if(child.getNodeName().equalsIgnoreCase("Description")){
                Node text = child.getFirstChild();
                descriptions.setText(id, text.getNodeValue());
                
             }
         }
    }
        
    
    public GeoPolygon parsePolygon(int id,Node node){
        Node parent = node.getParentNode();
        NamedNodeMap att= parent.getAttributes();
        
        GeoPolygon poly = null;
        System.out.println("Parsing Polygon "+id);
        NodeList children = node.getChildNodes();
         for(int i=0;i<children.getLength();i++){
             Node child = children.item(i);
             if(child.getNodeName().equalsIgnoreCase("LineString")){
                poly = new GeoPolygon(id,parseLineString(child));
             }
         }
        
        return poly;
    }
    
    public GeoPoint[] parseLineString(Node node){
        System.out.println("Parsing line string");
        NodeList children = node.getChildNodes();
        for(int i=0;i<children.getLength();i++){
             Node child = children.item(i);
             if(child.getNodeName().equalsIgnoreCase("CList")){
                return parseCList(child);
             }
         }
         return null;
        
    }
        
    
    public GeoPoint[] parseCList(Node node){
        GeoPoint point[] = null;
        Vector points = new Vector();
        System.out.println("Parsing CList");
        Node list = node.getFirstChild();
        StringTokenizer st = new StringTokenizer(list.getNodeValue(),", \n\t");
        while(st.hasMoreTokens()){
            double x = Double.valueOf(st.nextToken()).doubleValue();
            double y = Double.valueOf(st.nextToken()).doubleValue();
            GeoPoint p = new GeoPoint(x,y);
            points.addElement(p);
        }
       
        point = new GeoPoint[points.size()];
        points.copyInto(point);
        return point;
        
    }
    
    
    public static void main(String args[])throws MalformedURLException {
        GMLReader gml = new GMLReader(new URL("http://www.ccg.leeds.ac.uk/geotools/demos/gml/data/test2.xml"),null);
        System.out.println("Done");
    }
    
}
